import os
import time
import math
import subprocess
import bmesh
import re

from mathutils import Vector, Matrix

import importlib
import io_scene_fbx
from io_scene_fbx import export_fbx_bin
  
import bpy
from bpy.types import Header, Menu, Panel, Operator, AddonPreferences, PropertyGroup, Bone
#from bpy.props import CollectionProperty, StringProperty, BoolProperty, IntProperty, FloatProperty, FloatVectorProperty, EnumProperty, PointerProperty
from bpy.props import *

from bpy_extras.io_utils import (
        ImportHelper,
        ExportHelper,
        orientation_helper_factory,
        path_reference_mode,
        axis_conversion,
        )

def hge_add_prop(item, prop_name):
    def unique_name(names):
        prop = prop_name
        prop_new = prop
        i = 1
        while prop_new in names:
            prop_new = prop + str(i)
            i += 1

        return prop_new

    prop = unique_name(
            {*item.keys(),
                *type(item).bl_rna.properties.keys(),
                })

    item[prop] = 1.0
    rna_idprop_ui_prop_update(item, prop)

    # not essential, but without this we get [#31661]
    prop_ui = rna_idprop_ui_prop_get(item, prop)
    prop_ui["soft_min"] = prop_ui["min"] = 0.0
    prop_ui["soft_max"] = prop_ui["max"] = 1.0
    
def hge_prop_exists(object, prop):
    names = {*object.keys(), *type(object).bl_rna.properties.keys(),}
    while prop in names:
        return True
    return False

def hge_add_mtrl_props(material):
    if not hge_prop_exists(material, 'AlphaBlendMode'):
        material['AlphaBlendMode'] = 1
    if not hge_prop_exists(material, 'CMColors'):
        material['CMColors'] = 0
    if not hge_prop_exists(material, 'CMVersion'):
        material['CMVersion'] = 42
    if not hge_prop_exists(material, 'CastShadow'):
        material['CastShadow'] = 1
    if not hge_prop_exists(material, 'DepthSmooth'):
        material['DepthSmooth'] = 0
    if not hge_prop_exists(material, 'DepthWrite'):
        material['DepthWrite'] = 1
    if not hge_prop_exists(material, 'Special'):
        material['Special'] = 0
    if not hge_prop_exists(material, 'UseAlphaTest'):
        material['UseAlphaTest'] = 0
        #
        #
        #
        #
    if not hge_prop_exists(material, 'AmbientMapChannel'):
        material['AmbientMapChannel'] = ""
    if not hge_prop_exists(material, 'AmbientMapFile'):
        material['AmbientMapFile'] = ""
    if not hge_prop_exists(material, 'DiffuseMapChannel'):
        material['DiffuseMapChannel'] = ""
    if not hge_prop_exists(material, 'DiffuseMapFile'):
        material['DiffuseMapFile'] = ""
    if not hge_prop_exists(material, 'NormalMapChannel'):
        material['NormalMapChannel'] = ""
    if not hge_prop_exists(material, 'NormalMapFile'):
        material['NormalMapFile'] = ""
    if not hge_prop_exists(material, 'RoughnessMetallicMapChannel'):
        material['RoughnessMetallicMapChannel'] = ""
    if not hge_prop_exists(material, 'RoughnessMetallicMapFile'):
        material['RoughnessMetallicMapFile'] = ""
    if not hge_prop_exists(material, 'ColorizationMaskChannel'):
        material['ColorizationMaskChannel'] = ""
    if not hge_prop_exists(material, 'ColorizationMaskFile'):
        material['ColorizationMaskFile'] = ""
    if not hge_prop_exists(material, 'SelfIllumMapChannel'):
        material['SelfIllumMapChannel'] = ""
    if not hge_prop_exists(material, 'SelfIllumMapFile'):
        material['SelfIllumMapFile'] = ""
    if not hge_prop_exists(material, 'DustMapChannel'):
        material['SpecialMapChannel'] = ""
    if not hge_prop_exists(material, 'DustMapFile'):
        material['SpecialMapFile'] = ""

def hge_get_tex_uv_chan(texture):
    return texture.uv_layer

def hge_get_tex_file(texture):
    tex_filepath = texture.texture.image.filepath
    if os.path.exists(tex_filepath) and os.path.isabs(tex_filepath):
        #print("ABS TEX: " + texture.texture.image.filepath + "\n")
        return tex_filepath

    abs_filepat = tex_filepath
    if not os.path.isabs(abs_filepat):
        base_filepath = os.path.dirname(bpy.data.filepath);
        abs_filepat = os.path.normpath(base_filepath + "/" + abs_filepat)

    #print("CMB TEX: " + abs_filepat + "\n")
    if not os.path.exists(abs_filepat):
        bpy.ops.image.unpack(method='WRITE_LOCAL', id=texture.texture.image.name)
        if not os.path.exists(abs_filepat):
            base_filepath = os.path.dirname(bpy.data.filepath);
            tex_filepath = texture.texture.image.filepath
            abs_filepat = os.path.normpath(base_filepath + "/" + tex_filepath)

    if not os.path.exists(abs_filepat):
        print("\n\nERROR: TEXTURE DOESN'T EXIST: " + abs_filepat + "\n\n")
    else:
        texture.texture.image.filepath = abs_filepat

    return abs_filepat

def hge_is_texture_from_type(tex, type):
    tex_name = tex.name
    if tex_name == type:
       return True

    tokens = tex_name.split(".")
    if len(tokens) >= 2 and tokens[0] == type:
       return True

    return False

def hge_prepare_mtrl_for_exp(material):
    if not hge_prop_exists(material, 'AlphaBlendMode'):
        hge_add_mtrl_props(material)

    hgm_settings = material.hgm_settings

    material['AlphaBlendMode'] = int(hgm_settings.alpha_blend_mode)
    material['CMColors'] = int(hgm_settings.cm_colors)
    material['Special'] = int(hgm_settings.special)

    material['AnimationFramesX'] = hgm_settings.animation_frames_x
    material['AnimationFramesY'] = hgm_settings.animation_frames_y
    material['AnimationTime'] = hgm_settings.animation_time
    material['DepthSmooth'] = hgm_settings.depth_smooth

    material['CastShadow'] = hgm_settings.cast_shadows
    material['DepthWrite'] = hgm_settings.depth_write
    material['UseAlphaTest'] =  hgm_settings.use_alpha_test
    
    has_map = {'AmbientMapFile' : None, 
               'DiffuseMapFile' : None,
               'NormalMapFile' : None,
               'SelfIllumMapFile' : None,
               'RoughnessMetallicMapFile' : None,
               'ColorizationMaskFile' : None,
               'SpecialMapFile' : None}

    #Named texture pass
    for tex in material.texture_slots:
        if tex and tex.texture.type == "IMAGE":
            #also check by name
            if hge_is_texture_from_type(tex, "AO"):
                material['AmbientMapChannel'] = hge_get_tex_uv_chan(tex)
                material['AmbientMapFile'] = hge_get_tex_file(tex)
                has_map['AmbientMapFile'] = tex
            elif hge_is_texture_from_type(tex, "BaseColor"):
                material['DiffuseMapChannel'] = hge_get_tex_uv_chan(tex)
                material['DiffuseMapFile'] = hge_get_tex_file(tex)
                has_map['DiffuseMapFile'] = tex
            elif hge_is_texture_from_type(tex, "Normal"):
                material['NormalMapChannel'] = hge_get_tex_uv_chan(tex)
                material['NormalMapFile'] = hge_get_tex_file(tex)
                has_map['NormalMapFile'] = tex
            elif hge_is_texture_from_type(tex, "SelfIllum"):
                material['SelfIllumMapChannel'] = hge_get_tex_uv_chan(tex)
                material['SelfIllumMapFile'] = hge_get_tex_file(tex)
                has_map['SelfIllumMapFile'] = tex
            elif hge_is_texture_from_type(tex, "RoughnessMetallic"):
                material['RoughnessMetallicMapChannel'] = hge_get_tex_uv_chan(tex)
                material['RoughnessMetallicMapFile'] = hge_get_tex_file(tex)
                has_map['RoughnessMetallicMapFile'] = tex
            elif hge_is_texture_from_type(tex, "Colorization"):
                material['ColorizationMaskChannel'] = hge_get_tex_uv_chan(tex)
                material['ColorizationMaskFile'] = hge_get_tex_file(tex)
                has_map['ColorizationMaskFile'] = tex
            elif hge_is_texture_from_type(tex, "Dust"):
                material['SpecialMapChannel'] = hge_get_tex_uv_chan(tex)
                material['SpecialMapFile'] = hge_get_tex_file(tex)
                has_map['SpecialMapFile'] = tex

    #Type texture pass
    for tex in material.texture_slots:
        if tex and tex.texture.type == "IMAGE":
            chan_named = False
            type_recognized = False
            if tex.use_map_ambient:
                if has_map['AmbientMapFile'] == None:
                    material['AmbientMapChannel'] = hge_get_tex_uv_chan(tex)
                    material['AmbientMapFile'] = hge_get_tex_file(tex)
                    has_map['AmbientMapFile'] = tex
                    type_recognized = True
                elif has_map['AmbientMapFile'] == tex:
                    chan_named = True
            elif tex.use_map_color_diffuse:
                if has_map['DiffuseMapFile'] == None:
                    material['DiffuseMapChannel'] = hge_get_tex_uv_chan(tex)
                    material['DiffuseMapFile'] = hge_get_tex_file(tex)
                    has_map['DiffuseMapFile'] = tex
                    type_recognized = True
                elif has_map['DiffuseMapFile'] == tex:
                    chan_named = True
            elif tex.use_map_normal:
                if has_map['NormalMapFile'] == None:
                    material['NormalMapChannel'] = hge_get_tex_uv_chan(tex)
                    material['NormalMapFile'] = hge_get_tex_file(tex)
                    has_map['NormalMapFile'] = tex
                    type_recognized = True
                elif has_map['NormalMapFile'] == tex:
                    chan_named = True
            elif tex.use_map_emit:
                if has_map['SelfIllumMapFile'] == None:
                    material['SelfIllumMapChannel'] = hge_get_tex_uv_chan(tex)
                    material['SelfIllumMapFile'] = hge_get_tex_file(tex)
                    has_map['SelfIllumMapFile'] = tex
                    type_recognized = True
                elif has_map['SelfIllumMapFile'] == tex:
                    chan_named = True
            if chan_named == False and type_recognized == False:
                tex_is_named = False
                for key, value in has_map.items():
                    if value == tex:
                        tex_is_named = True
                        break
                if tex_is_named == False:
                    print("WARNING: Unable to find slot for texture: " + tex.name + " in material: " + material.name + "\n")
                    print("Make sure the texture is named according to HGE naming convention or influences a proper channel\n") 

    for key, value in has_map.items():
        if value == None:
            material[key] = ""

def hge_select_vertex(index):
    obj = bpy.context.active_object
    if obj.type != "MESH":
        return
    bpy.context.scene.objects.active = obj
    bpy.ops.object.mode_set.poll()
    bpy.ops.object.mode_set(mode='EDIT')
    me = obj.data
    bm = bmesh.from_edit_mesh(me)
    verts = bm.verts
    for vert in verts:
        if vert.index == index:
            vert.select = True
        else:
            vert.select = False
    bmesh.update_edit_mesh(me, True)

#hge_select_vertex(index=225)

class HGEAddMtrlProps(Operator):
    """Add HGE properties to the material"""
    bl_idname = "hge.add_mtrl_props"
    bl_label = "Add HGE properties to the material"

    def execute(self, context):
        objects = bpy.context.selected_objects
        for object in objects:
            if object.type == "MESH" :
               mat_slots = object.material_slots
               for slot in mat_slots:
                   mat = slot.material
                   if mat != None:
                      hge_add_mtrl_props(mat)
        return {'FINISHED'}

class HGEAddAllMtrlProps(Operator):
    """Add HGE properties to all materials in the scene"""
    bl_idname = "hge.add_all_mtrl_props"
    bl_label = "Add HGE properties to all materials in the scene"

    def execute(self, context):
        for mtrl in bpy.data.materials:
            hge_add_mtrl_props(mtrl)

        return {'FINISHED'}

#--------------------------------------------------------
class HGEMaterialSettings(PropertyGroup):
    alpha_blend_mode = EnumProperty(
            name="Alpha Blending Mode",
            description="HGE Alpha Blending Mode",
            items=(('1', "None", ""), 
                   ('2', "Blend", ""), 
                   ('3', "Additive", ""), 
                   ('4', "AdditiveLight", ""),
                   ('5', "Premultiplied", ""),
                   ('6', "Glass", "")), default='1',)

    cm_colors = EnumProperty(
            name="Number of masked colors in CM",
            description="HGE Number of masked colors in CM",
            items=(('1', "1", ""), 
                   ('2', "2", ""), 
                   ('3', "3", ""), 
                   ('4', "4", "")), default='1',)

    special = EnumProperty(
            name="Special",
            description="Special rendering mods, can be used to make the object a decal or use a special water shader.",
            
            items=(('1', "None", ""), 
                   ('2', "Water", ""), 
                   ('3', "Decal", "")),default='1',)

    depth_smooth = IntProperty(
            name="Depth Smooth (in cm)",
			description="Allows for smooth fading of object edges (decals only).",
            #min=0.000, max=1000000.0,
            default=0,
            )

    animation_frames_x = IntProperty(
            name="Animation Frames X",
			description="For animated textures: number of frames on each row.",
            #min=0.000, max=30.0,
            default=0,
            )

    animation_frames_y = IntProperty(
            name="Animation Frames Y",
			description="For animated textures: number of frames on each column.",
            #min=0.000, max=30.0,
            default=0,
            )
    animation_time = IntProperty(
            name="Animation Time (in ms)",
			description="The loop time of the animated texture, when this is set to more than 0, the texture is treated as animated.",
            #min=0.000, max=10000.0,
            default=0,
            )
    cast_shadows = BoolProperty(
            name="Cast Shadows",
            description="Setting this to false will cause the object to not cast shadows",
            default=True,
            )
    depth_write = BoolProperty(
            name="Depth Write",
            description="Determines if the object will be rendered in the depth buffer.",
            default=True,
            )
    use_alpha_test = BoolProperty(
            name="Use Alpha Test",
            description="Determines if the object should discard pixels where the alpha value is below a threshold.",
            default=False,
            )

class HGESettings(PropertyGroup):
    loop_anim = BoolProperty(
            name="HGE Loop Animation", 
            description="HGE Loop Animation", 
            default=False)
    compensate_z = BoolProperty(
            name="HGE Compensate Z", 
            description="HGE Compensate Z", 
            default=True)
    root_motion = EnumProperty(
            name="HGE Root Motion",
            description="HGE Root Motion",
            items=(('None', "None", ""), ('CME', "CME", ""), ('VDA', "VDA", ""), ('InverseCME', "InverseCME", "")), default='None',)
    mesh_name = StringProperty(
            name="HGE Mesh Name",
            description="HGE Mesh Name",
            default="", maxlen=128,
            )
    state_name = StringProperty(
            name="HGE State Name",
            description="HGE State Name",
            default="", maxlen=128,
            )
    global_scale = FloatProperty(
            name="Global Scale",
            min=0.000, max=1000000.0,
            default=1.0,
            )
    entity_name = StringProperty(
            name="HGE Entity Name",
            description="HGE Entity Name",
            default="", maxlen=128,
            )
    what_to_export = EnumProperty(
            name="What to export",
            description="What to export",
            items=(('Export All', "Export All", ""), ('Export Selected', "Export Selected", "")), default='Export All',)

def get_selected_bone_attribs(context, entity_name):
    obj = context.object

    if obj.type != "ARMATURE":
        return None, None

    # skeleton root format: hgs_entityName
    bones = obj.data.bones
    root_bone = None
    entity_name = ""
    for bone in bones:
        if bone:
            bone_name = bone.name
            tokens_list = bone_name.split(":")
            if len(tokens_list) >= 2:
                if tokens_list[0] == "hgs":
                    root_bone = bone
                    entity_name = tokens_list[1]
                    break

    if root_bone == None:
        return None, None

    return root_bone, entity_name

def hge_get_selected_bone(context):
    obj = context.object

    if obj.type != "ARMATURE":
        return None

    bones = obj.data.bones
    root_bone = None
    entity_name = ""
    for bone in bones:
        if bone and bone.select:
            return bone

    return None

def hge_bone_global_mat(bone):
    if not bone.parent:
        return bone.matrix
    else:
        return hge_bone_global_mat(bone.parent) * bone.matrix

def hge_print_selected_bone():
    obj = bpy.context.object
    if obj.type != "ARMATURE":
        return None
    bones = obj.pose.bones
    for bone in bones:
        if bone and bone.bone.select:
            #print((obj.matrix_world * hge_bone_global_mat(bone)).to_translation())
            #print(bone.matrix.to_translation() + obj.matrix_world.to_translation())
            print(bone.matrix.to_translation())
    return None

def hge_print_selected_bone2():
    obj = bpy.context.object
    if obj.type != "ARMATURE":
        return None
    bones = obj.data.bones
    for bone in bones:
        if bone and bone.select:
            print((obj.matrix_world * bone.matrix_local).to_translation())
    return None

class HGETagAnimation(Operator):
    """Mark HGE animation based on current timeline if properly named HGE skeleton root bone is selected"""
    bl_idname = "hge.tag_animation"
    bl_label = "Mark HGE animation based on current timeline if properly named HGE skeleton root bone is selected"

    def execute(self, context):

        scene = context.scene
        hge_settings = scene.hge_settings
        obj = context.object
        entity_name = hge_settings.entity_name
        state_name = hge_settings.state_name
        mesh_name = hge_settings.mesh_name
        root_motion = hge_settings.root_motion
        loop_anim = hge_settings.loop_anim
        compensate_z = hge_settings.compensate_z

        if not obj:
            return {'CANCELLED'}

        if not len(state_name):
            return {'CANCELLED'}

        if not len(mesh_name):
            return {'CANCELLED'}

        #anim format: hga:entityName:stateName:meshName
        #prop format: rootMotion_beginFrame_endFrame
        anim_name = "hga:" + entity_name + ":" + state_name + ":" + mesh_name
        root_bone = hge_get_selected_bone(context)

        if root_bone == None:
            return {'CANCELLED'}

        prop_name = root_motion
        prop_name += ":"
        prop_name += str(scene.frame_start)
        prop_name += ":"
        prop_name += str(scene.frame_end)
        prop_name += ":"
        #if scene.loop_anim != 0:
        #    prop_name += "True"
        #else:
        #    prop_name += "False"
        #prop_name += ":"
        #if scene.compensate_z != 0:
        #    prop_name += "True"
        #else:
        #    prop_name += "False"
        prop_name += str(loop_anim)
        prop_name += ":"
        prop_name += str(compensate_z)
        root_bone[anim_name] = prop_name

        xport_anim_name = "hgx:" + entity_name + ":" + state_name + ":" + mesh_name
        root_bone[xport_anim_name] = True
        return {'FINISHED'}
#--------------------------------------------------------
def hge_find_anim_range(objects, context):
    scene = context.scene
    min_frame = scene.frame_start
    max_frame = scene.frame_end
    for object in objects:
        if object.type == "ARMATURE":
            bones = object.data.bones
            for bone in bones:
                keys = bone.keys()
                #anim format: hga:entityName:stateName:meshName
                #prop format: rootMotion_beginFrame_endFrame
                for key in keys:
                    tokens = key.split(":")
                    if tokens[0] == "hga" and len(tokens) == 4:
                        #print("found anim " + key + "\n")
                        prop = bone[key]
                        #print("prop " + prop + "\n")
                        tokens_prop = prop.split(":")
                        if len(tokens_prop) == 3:
                            begin_frame = int(tokens_prop[1])
                            end_frame = int(tokens_prop[2])
                            if min_frame > begin_frame:
                                min_frame = begin_frame
                            if max_frame < end_frame:
                                max_frame = end_frame
    return min_frame, max_frame

def hge_is_attach(name):
    tokens = name.split(" ")
    tok_cnt = len(tokens)
    if(tok_cnt > 0 and len(tokens[0]) > 0):
        firs_tok = tokens[0]
        if firs_tok[0] == "-":
            return True
    return False

def hge_is_surface_type(name):
    surface_types = {'blockpass', 'terrain', 'build', 'flat', 'collision', 'hex_shape', 'service', 
    'minheight', 'maxheight', 'terrain_hole', 'height', 'walk', 'selection'}
    name_split = name.split(".")
    if((len(name_split) > 0) and (len(name_split[0]) > 0) and (name_split[0].lower() in surface_types)):
        return True
    else:
        return False

def hge_setup_waypoint(object, parent, chain, waypoints):
    print("Selected obj type: " + object.type + "\n")
    print("Splines count: " + str(len(object.data.splines)) + "\n")
    for spline in object.data.splines:
        index = 0
        print("Points count: " + str(len(spline.points)))
        for point in spline.points:
            index += 1
            loc = point.co
            #print("Location: " + str(loc))
            mat = object.matrix_world * parent.matrix_world.inverted()
            loc = mat * loc
            loc3d = (loc[0], loc[1], loc[2])
            bpy.ops.object.empty_add(location=loc3d)
            waypt = bpy.data.objects[bpy.context.active_object.name]
            waypt.parent = parent
            object_name = object.name

            regex_result = re.match("^-(.*?)(\.\d\d\d)?$", object_name, flags=0)
            if regex_result:
                object_name = "-" + regex_result.group(1)
            
            if hge_prop_exists(object, "attach"):
                object_name += str(object["attach"])

            words = object_name.split(";")
            #print("Words count: " + str(len(words)))
            waypt.name = words[0] + ";"
            name = ""

            if(index == 1):
                for j in range(1, len(words)):
                    name += words[j] + ","

            name += "chain=" + str(chain) + ",waypoint=" + str(index)
            waypt["attach"] = name
            waypoints.append(waypt)

def hge_setup_waypoints(objects, context):
    scene = context.scene
    waypoints = []
    attaches = []
    chain = 0
    for object in objects:
        if object.type == "MESH":
            children = object.children
            for child in children:
                if hge_is_attach(child.name) and child.type == "CURVE":
                    chain += 1
                    hge_setup_waypoint(child, object, chain, waypoints)
                    hide_attach_name = child.name[1:]
                    print("Hide attach name: " + hide_attach_name + "\n")
                    child.name = hide_attach_name
                    attaches.append(child)
       
    return waypoints, attaches

def hge_find_objects_for_export(objects, selected, meshes, anims):
    exported = set()
    animated_entities = set()
    entity_meshes = {}
    exported_meshes = set()
    for obj in objects:
        #bpy.context.scene.objects.active.modifiers[0].type == "ARMATURE"
        #if obj.type == "ARMATURE":
        #    exported.add(obj)
        if obj.type == "MESH":
            words = obj.name.split(":")
            if len(words) >= 1 and words[0] == "hgm":
                
                if not (words[1] in entity_meshes):
                    entity_meshes[words[1]] = []
                entity_meshes[words[1]].append(obj)

                if meshes and (not selected or obj.hge_export):
                    exported.add(obj)
                    exported_meshes.add(obj)
                    obj.hge_export = True
                else:
                    obj.hge_export = False

        elif obj.type == "ARMATURE" and anims:
            bones = obj.data.bones
            for bone in bones:
                if bone:
                    bone_name = bone.name
                    items = bone.items()
                    items.sort()

                    for key, val in items:
                        tokens_list = key.split(":")
                        if len(tokens_list) >= 4 and tokens_list[0] == "hga":
                            #anim format: hga:entityName:stateName:meshName
                            #prop format: rootMotion:beginFrame:endFrame
                            entity_name = tokens_list[1]
                            state_name = tokens_list[2]
                            mesh_name = tokens_list[3]

                            tokens_prop = val.split(":")

                            xport_anim_name = "hgx:" + entity_name + ":" + state_name + ":" + mesh_name

                            if not selected or not hge_prop_exists(bone, xport_anim_name) or bone[xport_anim_name] == 1:
                                exported.add(obj)
                                animated_entities.add(entity_name)
                                bone[xport_anim_name] = True
                            else:
                                bone[xport_anim_name] = False

    #add all meshes from animated entities
    for entity in animated_entities:
        for ent_mesh in entity_meshes[entity]:
            exported.add(ent_mesh)
            exported_meshes.add(ent_mesh)

    #check and add attaches for each mesh
    for mesh in exported_meshes:
        #print ("exported mesh: " + mesh.name + "\n")
        children = mesh.children
        for child in children:
            #print ("child: " + child.name + "\n")
            if hge_is_attach(child.name) or hge_is_surface_type(child.name):
                exported.add(child)

    #add all armatures for exported meshes
    for mesh in exported_meshes:
        for mod in mesh.modifiers:
            if mod.type == "ARMATURE" and mod.object != None:
                armature_obj = mod.object
                exported.add(armature_obj)
                #add all attaches from bones
                children = armature_obj.children
                for child in children:
                    if child.parent_bone != None and len(child.parent_bone) > 0 and hge_is_attach(child.name) or hge_is_surface_type(child.name):
                        exported.add(child)

    #add all parents
    parents = set()
    for item in exported:
        parent = item.parent
        while parent:
            parents.add(parent)
            parent = parent.parent

    exported = exported.union(parents)
    return exported

def hge_set_export_mode(objects):
    for obj in objects:
        bpy.context.scene.objects.active = obj
        bpy.ops.object.mode_set.poll()
        if obj.type == "ARMATURE":
            bpy.ops.object.mode_set(mode='POSE')
        else:
            bpy.ops.object.mode_set(mode='OBJECT')



def hge_export(operator, context, selected, meshes, anims):
    global_scale = 1.0
    axis_up = "Z"
    axis_forward = "Y"

    scene = context.scene
    global_matrix = (Matrix.Scale(global_scale, 4) *
                        axis_conversion(to_forward=axis_forward,
                                        to_up=axis_up,
                                        ).to_4x4())

    filename, ext = os.path.splitext(os.path.basename(bpy.data.filepath))
    directory = os.getenv('APPDATA') + "/Surviving Mars/ModAssets/FBX"
    filepath = os.path.normpath(directory + "/" + filename + ".fbx")

    if not os.path.exists(directory):
        os.makedirs(directory)

    active_object = context.scene.objects.active
    org_mode = None
    if active_object and active_object.mode != 'OBJECT' and bpy.ops.object.mode_set.poll():
        org_mode = active_object.mode
    #    bpy.ops.object.mode_set(mode='OBJECT')

    ctx_objects = context.scene.objects

    #find the time range for all animation
    saved_start_frame = scene.frame_start
    saved_end_frame = scene.frame_end
    min_frame, max_frame = hge_find_anim_range(ctx_objects, context)
    scene.frame_start = min_frame
    scene.frame_end = max_frame

    #generate the additional waypoints from the dummy attaches. then rename a dummy attaches 
    waypoints, attaches = hge_setup_waypoints(ctx_objects, context)

    #find and export only the tagged objects
    ctx_objects = context.scene.objects
    exp_objects = hge_find_objects_for_export(ctx_objects, selected, meshes, anims)
    #for exp_obj in exp_objects:
    #    print ("exporting obj: " + exp_obj.name + "\n")

    #prepare all selected materials for export. Only for tagged meshes
    for obj in exp_objects:
        if(obj.type == "MESH"):
            mtokens = obj.name.split(":")
            if(len(mtokens) >= 5 and mtokens[0] == "hgm"):
                mat_slots = obj.material_slots
                has_materials = False
                for slot in mat_slots:
                    if(slot.material != None):
                        hge_prepare_mtrl_for_exp(slot.material)
                        has_materials = True

                #there is no materials for this mesh. add an default material and try again
                if has_materials == False:
                    mat = bpy.data.materials.new(name="Material")
                    obj.data.materials.append(mat)
                    mat_slots = obj.material_slots
                    for slot in mat_slots:
                        if(slot.material != None):
                            hge_prepare_mtrl_for_exp(slot.material)

    hge_set_export_mode(exp_objects)
            
    export_fbx_bin.save_single(operator, scene, filepath=filepath,
            global_matrix=global_matrix,
            apply_unit_scale=False,
            axis_up="Z",
            axis_forward="Y",
            context_objects=exp_objects,
            object_types={'ARMATURE', 'OTHER', 'MESH', 'EMPTY'},
            use_mesh_modifiers=True,
            mesh_smooth_type='OFF',
            use_armature_deform_only=False,
            bake_anim=True,
            bake_anim_use_all_bones=True,
            bake_anim_use_nla_strips=False,
            bake_anim_use_all_actions=False,
            bake_anim_step=1.0,
            bake_anim_simplify_factor=1.0,
            bake_anim_force_startend_keying=True,
            add_leaf_bones=False,
            primary_bone_axis='Y',
            secondary_bone_axis='X',
            use_metadata=True,
            path_mode='AUTO',
            use_mesh_edges=False,
            use_tspace=False,
            embed_textures=False,
            use_custom_props=True,
            bake_space_transform=False,
            armature_nodetype='NULL',)

    scene.frame_start = saved_start_frame
    scene.frame_end = saved_end_frame
    
    #delete the generated additional waypoints
    print("Waypoints count: " + str(len(waypoints)) + "\n") 
    for wp in waypoints:
        #print("WP: " + wp.name)
        bpy.context.scene.objects.active = wp
        wp.select = True
        bpy.ops.object.delete()

    #restore names for dummy attaches
    for attach in attaches:
        aname = attach.name
        attach.name = "-" + aname

    context.scene.objects.active = active_object
    if active_object and org_mode and bpy.ops.object.mode_set.poll():
        bpy.ops.object.mode_set(mode=org_mode)

    #start AssetProcessor with parser
    #------------------------------------------------------------
    assets_proc_path = os.getenv('HGEAP')
    if assets_proc_path == None or not os.path.exists(assets_proc_path):
        print ("Checking for AssetsProcessor in game directory\n")
        reg_path_val = subprocess.check_output("reg query \"HKEY_CURRENT_USER\SOFTWARE\Haemimont Games\Surviving Mars\" /v Path", stderr=subprocess.STDOUT).decode("ascii")
        reg_path_val = re.search(r"REG_SZ\s*(.*)\\", reg_path_val)
        assets_proc_path = reg_path_val.group(1) + r"\ModTools\AssetsProcessor\AssetsProcessor.exe"

    assets_proc_path = os.path.normpath(assets_proc_path)
    asset_prop_args = "\"\"" + assets_proc_path + "\" \"" + filepath + "\" -globalappdirs\"";
    
    print("asset_prop_args: " + asset_prop_args + "\n")
    os.system(asset_prop_args)

    return {'FINISHED'}

class HGECreateWaypoints(Operator):
    """Create waypoints from path"""
    bl_idname = "hge.create_waypoints"
    bl_label = "Create waypoints from path"

    def execute(self, context):
        chain = 1
        waypoints = []
        active_object = context.scene.objects.active
        hge_setup_waypoint(active_object, active_object.parent, chain, waypoints)
        return {'FINISHED'}

class HGERemoveAnm(Operator):
    """Remove animation for given entity"""
    bl_idname = "hge.remove_anim"
    bl_label = "Remove Anim"

    armature = StringProperty("")
    bone = StringProperty("")
    property = StringProperty("")

    def execute(self, context):
        object = bpy.data.objects[self.armature]
        bone = object.data.bones[self.bone]
        del bone[self.property]
        
        return {'FINISHED'}


class HGEToggleMeshExport(Operator):
    """Toggle mesh for export"""
    bl_idname = "hge.toggle_mesh_export"
    bl_label = "Toggle mesh export"

    mesh_name = StringProperty("")
    
    def execute(self, context):
        mesh = context.scene.objects[self.mesh_name]

        if hge_prop_exists(mesh, 'export'):
            if mesh['export']==1:
                mesh['export'] = 0.0
            else:
                mesh['export'] = 1.0
        else:
            mesh['export'] = 1.0

        self.report({'INFO'},"Mesh report")

        return {'FINISHED'}

class HGEToggleAnimExport(Operator):
    """Toggle animation for export"""
    bl_idname = "hge.toggle_anim_export"
    bl_label = "Toggle animation for export"

    armature = StringProperty("")
    bone = StringProperty("")
    property = StringProperty("")
    #anim_export = BoolProperty()

    def execute(self, context):
        object = context.scene.objects[self.armature]
        bone = object.data.bones[self.bone]
        
        #if(self.anim_export == True):
        if hge_prop_exists(bone, self.property):
            if bone[self.property]==True:
                bone[self.property] = False
            else:
                bone[self.property] = True
        else:
            bone[self.property] = True

        self.report({'INFO'},"Anim report")

        return {'FINISHED'}

#-------------------------------------------------

class AnimExportProperty(bpy.types.PropertyGroup):
    
    animation = StringProperty(name="Animation Name")
    armature = StringProperty(name="Armature Name")
    bone = StringProperty(name="Bone Name")
    property = StringProperty(name="Property Name")
    export = BoolProperty(name="Export", description="Exporet this animation", default=True)

class MeshExportProperty(bpy.types.PropertyGroup):
    
    #mesh = StringProperty(name="Mehs Name")
    #lod = StringProperty(name="LOD")
    #object = StringProperty(name="Object Name")
    #property = StringProperty(name="Property Name")
    item_mesh = StringProperty(name="Item Name")
    entity = StringProperty(name="Entity Name")
    mesh = StringProperty(name="Mehs Name")
    lod = StringProperty(name="LOD")
    export = BoolProperty(name="Export", description="Exporet this mesh", default=True)
    #objects = set()


class HGEExportDialog(Operator):
    """Export Dialog"""
    bl_idname = "hge.export_dialog"
    bl_label = "Export Dialog"
    
    animations = bpy.props.CollectionProperty(name="Animations", type=AnimExportProperty, description="Select animations to exporet")
    entity_meshes = bpy.props.CollectionProperty(name="Meshes", type=MeshExportProperty, description="Select meshes to exporet")
    entity_mesh_objects = {}

    exp_meshes = BoolProperty(default=True)
    exp_anims = BoolProperty(default=True)

    def invoke(self, context, event):
        self.animations.clear()
        self.entity_meshes.clear()
        #--------------------------------------------------
        #http://blender.stackexchange.com/questions/1779/dynamic-creation-of-properties-for-export-script
        #add anim properties for each animation
        scene = context.scene
        for obj in scene.objects:
            if obj.type == "MESH":
                #format: hgm:entityName:meshName:LOD:LODDistance:dyn:
                tokens_list = obj.name.split(":")
                if len(tokens_list) >= 5 and tokens_list[0] == "hgm":
                    ent_mesh = None
                    for mindx in range(len(self.entity_meshes)):
                        cur_ent_mesh = self.entity_meshes[mindx]
                        if cur_ent_mesh.entity == tokens_list[1] and cur_ent_mesh.mesh == tokens_list[2] and cur_ent_mesh.lod == tokens_list[3]:
                            ent_mesh = cur_ent_mesh
                            break
                    
                    if ent_mesh == None:
                        ent_mesh = self.entity_meshes.add()
                        label_name = "MESH:"
                        label_name += " Entity="
                        label_name += tokens_list[1]
                        label_name += ", Mesh="
                        label_name += tokens_list[2]
                        label_name += ", Lod="
                        label_name += tokens_list[3]
                        ent_mesh.item_mesh = label_name
                        ent_mesh.entity = tokens_list[1]
                        ent_mesh.mesh = tokens_list[2]
                        ent_mesh.lod = tokens_list[3]
                    
                    entity_mesh_name = tokens_list[1]
                    entity_mesh_name += ":"
                    entity_mesh_name += tokens_list[2]
                    entity_mesh_name += ":"
                    entity_mesh_name += tokens_list[3]

                    if not (entity_mesh_name in self.entity_mesh_objects):
                        self.entity_mesh_objects[entity_mesh_name] = set()

                    self.entity_mesh_objects[entity_mesh_name].add(obj)

            elif obj.type == "ARMATURE":
                bones = obj.data.bones
                for bone in bones:                    
                    items = bone.items()
                    items.sort()

                    for key, val in items:
                        tokens_list = key.split(":")
                        if len(tokens_list) >= 4 and tokens_list[0] == "hga":
                            #anim format: hga:entityName:stateName:meshName
                            #prop format: rootMotion:beginFrame:endFrame
                            entity_name = tokens_list[1]
                            state_name = tokens_list[2]
                            mesh_name = tokens_list[3]

                            tokens_prop = val.split(":")

                            root_motion = tokens_prop[0]
                            begin_frame = tokens_prop[1]
                            end_frame = tokens_prop[2]

                            xport_anim_name = "hgx:" + entity_name + ":" + state_name + ":" + mesh_name

                            if not hge_prop_exists(bone, xport_anim_name):
                                bone[xport_anim_name]=True

                            anim = self.animations.add()

                            label_name = "ANIMATION:"
                            label_name += state_name

                            label_name += ", entity="
                            label_name += entity_name
                            label_name += ", mesh="
                            label_name += mesh_name
                            
                            label_name += ", motion="
                            label_name += root_motion
                            label_name += ", start="
                            label_name += begin_frame
                            label_name += ", end="
                            label_name += end_frame

                            anim.animation = label_name
                            anim.armature = obj.name
                            anim.bone = bone.name
                            anim.property = xport_anim_name
                            anim.export = bone[xport_anim_name]
        #--------------------------------------------------
        wm = context.window_manager
        return wm.invoke_props_dialog(self, width=800, height=600)
        #return wm.invoke_popup(self, width=800, height=600)
    
    def execute(self, context):

        scene = context.scene
        for aindx in range(len(self.animations)):
            animation = self.animations[aindx]
            armature = scene.objects[animation.armature]
            bone = armature.data.bones[animation.bone]
            bone[animation.property] = animation.export

        self.animations.clear()

        for mindx in range(len(self.entity_meshes)):
            ent_mesh = self.entity_meshes[mindx]
           
            entity_mesh_name = ent_mesh.entity
            entity_mesh_name += ":"
            entity_mesh_name += ent_mesh.mesh
            entity_mesh_name += ":"
            entity_mesh_name += ent_mesh.lod

            for obj in self.entity_mesh_objects[entity_mesh_name]:
                obj.hge_export = ent_mesh.export

        self.entity_meshes.clear()

        hge_settings = scene.hge_settings
        print ("what_to_export: " + str(hge_settings.what_to_export) + "\n")
        if hge_settings.what_to_export == "Export Selected":
            hge_export(self, context, True, self.exp_meshes, self.exp_anims)
        else:
            hge_export(self, context, False, self.exp_meshes, self.exp_anims)

        return {'FINISHED'}
    
    def draw(self, context):
        print ("REDRAW EXPORT DLG\n")
        layout = self.layout
        row = layout.row()
        row.label("ITEMS:")
        scene = context.scene
        hge_settings = scene.hge_settings
        if self.exp_meshes:
            for mindx in range(len(self.entity_meshes)):
                ent_mesh = self.entity_meshes[mindx]
                objects_str = " ( "
                
                first = True

                entity_mesh_name = ent_mesh.entity
                entity_mesh_name += ":"
                entity_mesh_name += ent_mesh.mesh
                entity_mesh_name += ":"
                entity_mesh_name += ent_mesh.lod

                for obj in self.entity_mesh_objects[entity_mesh_name]:
                    #format: hgm:entityName:meshName:LOD:LODDistance:dyn:
                    tokens_list = obj.name.split(":")
                    tokens_len = len(tokens_list)
                    if tokens_len >= 5 and tokens_list[0] == "hgm":
                        if tokens_len >= 6 and "s=" in tokens_list[5] and tokens_len >= 7:
                            if not first:
                                objects_str += ", "
                            first = False
                            objects_str += tokens_list[6]
                        elif tokens_len >= 6:
                            if not first:
                                objects_str += ", "
                            first = False
                            objects_str += tokens_list[5]
                objects_str += ")"

                label_text = ent_mesh.item_mesh + objects_str
                row = layout.row()
                row.prop(ent_mesh, "export", text=label_text)

        if self.exp_anims:
            for aindx in range(len(self.animations)):
                row = layout.row()
                row.prop(self.animations[aindx], "export", text=self.animations[aindx].animation)

        row = layout.row(align=True)

        row = layout.row(align=True)
        row.prop(hge_settings, "what_to_export", expand=True)
#-------------------------------------------------

class HGEToolBar:
    bl_label = "HGE Tools"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'TOOLS'
    
    bl_category = "HGE Tools"

    def draw(self, context):
        layout = self.layout

        scene = context.scene
        hge_settings = scene.hge_settings
        
        obj = context.object

        root_bone = hge_get_selected_bone(context)
        row = layout.row()
        version_str = "VERSION: "
        version_str += scene.hge_version
        row.label(version_str)
        row = layout.row()
        row.label("ANIMATIONS:")
        row = layout.row(align=True)
        row.prop(hge_settings, "entity_name", text="Entity Name")
        row = layout.row(align=True)
        row.prop(hge_settings, "mesh_name", text="Mesh Name")
        row = layout.row(align=True)
        row.prop(hge_settings, "state_name", text="State Name")
        row = layout.row(align=True)
        row.prop(hge_settings, "root_motion", text="Root Motion")
        row = layout.row(align=True)
        row.prop(hge_settings, "loop_anim", text="Loop Animation")
        row = layout.row(align=True)
        row.prop(hge_settings, "compensate_z", text="Compensate Z")
        row = layout.row(align=True)
        add_anm_text = "Add New: "
        valid_attribs = True;
        if(root_bone != None):
            add_anm_text += "root=";
            add_anm_text += root_bone.name
        else:
            add_anm_text += "[Select Root Bone]";
            valid_attribs = False
        if len(hge_settings.entity_name) > 0:
            add_anm_text += ", entity=";
            add_anm_text += hge_settings.entity_name
        else:
            add_anm_text += ", [Enter Entity Name]";
            valid_attribs = False
        if len(hge_settings.mesh_name) > 0:
            add_anm_text += ", mesh=";
            add_anm_text += hge_settings.mesh_name
        else:
            add_anm_text += ", [Enter Mesh Name]";
            valid_attribs = False;
        if len(hge_settings.state_name) > 0:
            add_anm_text += ", state="
            add_anm_text += hge_settings.state_name
        else:
            add_anm_text += ", [Enter State Name]"
            valid_attribs = False;

        if valid_attribs == True:
            add_anm_text += ", motion="
            add_anm_text += hge_settings.root_motion

            #add_anm_text += ", loop="
            #add_anm_text += str(hge_settings.loop_anim)

            #add_anm_text += ", compZ="
            #add_anm_text += str(hge_settings.compensate_z)

        row.operator("hge.tag_animation", text=add_anm_text)

        #---------------------------------------
        if root_bone != None:
            items = root_bone.items()
            items.sort()

            for key, val in items:
                tokens_list = key.split(":")
                if len(tokens_list) >= 4 and tokens_list[0] == "hga":
                    #anim format: hga:entityName:stateName:meshName
                    #prop format: rootMotion:beginFrame:endFrame
                    tokens_prop = val.split(":")

                    row = layout.row()
                    box = row.box()
                    split = box.split(percentage=0.90)
                    row = split.row()
                    
                    anim_name = "entity="
                    anim_name += tokens_list[1]
                    anim_name += ", mesh="
                    anim_name += tokens_list[3]
                    anim_name += ", state="
                    anim_name += tokens_list[2]
                    anim_name += ", motion="
                    anim_name += tokens_prop[0]
                    anim_name += ", start="
                    anim_name += tokens_prop[1]
                    anim_name += ", end="
                    anim_name += tokens_prop[2]
                    if len(tokens_prop) >= 4:
                        anim_name += ", loop="
                        anim_name += tokens_prop[3]
                    if len(tokens_prop) >= 5:
                        anim_name += ", compz="
                        anim_name += tokens_prop[4]

                    row.label(anim_name)
                    row = split.row(align=True)
                    row.alignment = 'RIGHT'
                    props = row.operator("hge.remove_anim", text="", icon='ZOOMOUT')
                    props.armature = obj.name
                    props.bone = root_bone.name
                    props.property = key
        #---------------------------------------

        row = layout.row(align=True)
        row.label("EXPORT:")
       
        row = layout.row(align=True)
        prop = row.operator("hge.export_dialog", text="Export")
        prop.exp_meshes = True
        prop.exp_anims = True

        row = layout.row(align=True)
        prop = row.operator("hge.export_dialog", text="Export Meshes")
        prop.exp_meshes = True
        prop.exp_anims = False

        row = layout.row(align=True)
        prop = row.operator("hge.export_dialog", text="Export Animations")
        prop.exp_meshes = False 
        prop.exp_anims = True

# So we can have a panel in both object mode and editmode
class HGEToolBarObject(Panel, HGEToolBar):
    bl_idname = "MESH_PT_hge_object"
    bl_context = "objectmode"

class HGEToolBarPose(Panel, HGEToolBar):
    bl_idname = "MESH_PT_hge_pose"
    bl_context = "posemode"

class HGEToolBarMesh(Panel, HGEToolBar):
    bl_idname = "MESH_PT_hge_mesh"
    bl_context = "mesh_edit"

def hge_draw_mat(self, context):
    layout = self.layout
    ob = context.object
    hgm_settings = ob.active_material.hgm_settings
    layout.label("HAEMIMONT MATERIAL:")
    layout.prop(hgm_settings, "alpha_blend_mode")
    layout.prop(hgm_settings, "cm_colors")
    layout.prop(hgm_settings, "special")

    layout.prop(hgm_settings, "depth_smooth")
    layout.prop(hgm_settings, "animation_frames_x")
    layout.prop(hgm_settings, "animation_frames_y")
    layout.prop(hgm_settings, "animation_time")

    layout.prop(hgm_settings, "cast_shadows")
    layout.prop(hgm_settings, "depth_write")
    layout.prop(hgm_settings, "disable_dtm")
    layout.prop(hgm_settings, "normal_map_ad_distortion")
    layout.prop(hgm_settings, "use_alpha_test")

def hge_draw_tex(self, context):
    layout = self.layout
    ob = context.object
    active_tex_idx = ob.active_material.active_texture_index
    tslot = ob.active_material.texture_slots[active_tex_idx]
    #hgt_settings = tslot.hgt_settings
    #layout.prop(hgt_settings, "texture_type")
    layout.prop(tslot, "hge_texture_type")
    #hgm_settings = ob.active_material.hgm_settings
    #layout.prop(ob.active_material.hgm_settings, "my_float")

def register():
    
    hge_version = "1.0"
    trunk_root = os.getenv('HGETrunkRoot')
    if trunk_root != None:
        svn_arg = "svnversion -n " + trunk_root
        svn_out = subprocess.check_output(svn_arg, stderr=subprocess.STDOUT).decode("ascii")
        print("svn_out: " + svn_out + "\n")
        hge_version = svn_out
    else:
        reg_path_val = subprocess.check_output("reg query \"HKEY_CURRENT_USER\SOFTWARE\Haemimont Games\Surviving Mars\" /v Path", stderr=subprocess.STDOUT).decode("ascii")
        reg_path_val = re.search(r"REG_SZ\s*(.*)\\", reg_path_val)
        if reg_path_val:
            mod_tools_path = reg_path_val.group(1) + r"\ModTools\LuaRevision.txt"
            if os.path.exists(mod_tools_path):
                with open(mod_tools_path) as f:
                    for line in f:
                        hge_version = line
                        break

    bpy.utils.register_class(HGEToolBarObject)
    bpy.utils.register_class(HGEToolBarPose)
    bpy.utils.register_class(HGEToolBarMesh)
    bpy.utils.register_class(HGEAddMtrlProps)
    bpy.utils.register_class(HGEAddAllMtrlProps)
    bpy.utils.register_class(HGETagAnimation)
    bpy.utils.register_class(HGESettings)
    bpy.utils.register_class(HGEMaterialSettings)
    bpy.utils.register_class(HGECreateWaypoints)
    #bpy.utils.register_class(HGETextureSettings)
    bpy.utils.register_class(HGERemoveAnm)
    bpy.utils.register_class(HGEToggleMeshExport)
    bpy.utils.register_class(HGEToggleAnimExport)
    bpy.utils.register_class(AnimExportProperty)
    bpy.utils.register_class(MeshExportProperty)
    bpy.utils.register_class(HGEExportDialog)
    bpy.types.Scene.hge_settings = PointerProperty(type=HGESettings)
    bpy.types.Scene.hge_version = StringProperty(
            name="HGE Version",
            description="HGE Version",
            default=hge_version, maxlen=128,
            )
    #bpy.types.MaterialTextureSlot.hgt_settings = PointerProperty(type=HGETextureSettings)
    bpy.types.Material.hgm_settings = PointerProperty(type=HGEMaterialSettings)
    bpy.types.Object.hge_export = BoolProperty(name="hge_export", default=True,)
    bpy.types.MATERIAL_PT_context_material.append(hge_draw_mat)
    #bpy.types.TEXTURE_PT_context_texture.append(hge_draw_tex)

def unregister():
    bpy.types.MATERIAL_PT_context_material.remove(hge_draw_mat)
    #bpy.types.TEXTURE_PT_context_texture.remove(hge_draw_tex)
    del bpy.types.Scene.hge_settings
    del bpy.types.Scene.hge_version
    del bpy.types.Material.hgm_settings
    del bpy.types.Object.hge_export
    #del bpy.types.MaterialTextureSlot.hgt_settings

    bpy.utils.unregister_class(HGEToolBarObject)
    bpy.utils.unregister_class(HGEToolBarPose)
    bpy.utils.unregister_class(HGEToolBarMesh)
    bpy.utils.unregister_class(HGEAddMtrlProps)
    bpy.utils.unregister_class(HGEAddAllMtrlProps)
    bpy.utils.unregister_class(HGETagAnimation)
    bpy.utils.unregister_class(HGESettings)
    bpy.utils.unregister_class(HGEMaterialSettings)
    bpy.utils.unregister_class(HGECreateWaypoints)
    #bpy.utils.unregister_class(HGETextureSettings)
    bpy.utils.unregister_class(HGERemoveAnm)
    bpy.utils.unregister_class(HGEToggleMeshExport)
    bpy.utils.unregister_class(HGEToggleAnimExport)
    bpy.utils.unregister_class(HGEExportDialog)
    bpy.utils.unregister_class(AnimExportProperty)
    bpy.utils.unregister_class(MeshExportProperty)
